/**
 * Help screen for the lockpicking minigame with assorted button.
 * @param {string} id Scene id. Used to create controllers.
 * @param {string} key Unique key-space for the lockpicking minigame.
 * @param {LockPickingMinigame} master The master that created this component. 
 */
function LockPickingLockPick(id, key, master) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier;
    var ctrl = new ChoiceController(id, elementId);
    var image = new LockPickingPreloadingLockPick();

    var amountOfClickBoxes = 5;
    var inTransition = false;
    var tapDelay = 300;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Called when a CSS transition ends.
     */
    function transitionEnd() {
        inTransition = false;
        $("#" + elementId).removeClass("lockpicktap");
    }

    /**
     * Plays the tap animation.
     */
    function tapAnimation() {
        inTransition = true;
        $("#" + elementId).addClass("lockpicktap");
    }

    /**
     * Turns the lockpick's position into a CSS class.
     * @returns {string} The CSS class.
     */
    function positionToClass() {
        return "lockpickpos" + that.position;
    }

    /**
     * Swaps position classes.
     * @param {string} oldClass Old position class.
     * @param {string} newClass New position class.
     */
    function swapClasses(oldClass, newClass) {
        inTransition = true;
        $("#" + elementId).removeClass(oldClass);
        $("#" + elementId).addClass(newClass);
    }

    /**
     * Attaches a listener to the box with the given rank number.
     * @param {int} number A number from 0 to 4 denoting the box.
     */
    function listenerFor(number) {
        $("#" + elementId + number).click(function () {
            if (that.position === number) {
                that.tap();
            } else {
                that.moveToPos(number);
            }
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    this.position = (ctrl.has() && ctrl.get()) || 0;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML.
     */
    this.build = function () {
        var lockpick = `<img src=${image.url} id="${elementId}" class="lockpick ${positionToClass()}">`;
        var boxes = ``;
        for (var i = 0; i < amountOfClickBoxes; i++) {
            boxes += `<div class="clickbox clickbox${i}" id="${elementId + i}"></div>`;
        }
        return lockpick + boxes;
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {
        $("#" + elementId).on("transitionend MSTransitionEnd msTransitionEnd webkitTransitionEnd oTransitionEnd animationend oAnimationEnd webkitAnimationEnd", transitionEnd);
        for (var i = 0; i < amountOfClickBoxes; i++) {
            listenerFor(i);
        }
    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        return image.preload();
    };

    /**
     * Taps the lockpick.
     */
    this.tap = function () {
        if (inTransition) {
            return;
        }
        tapAnimation();
        setTimeout(function () {
            master.springsAndTumblers.up(that.position);
        }, tapDelay);
    };

    /**
     * Makes the lockpick break. Animation only.
     */
    this.break = function () {
        inTransition = true;
        $("#" + elementId).addClass("lockpickpos-1");
        setTimeout(function() {
            $("#" + elementId).removeClass("lockpickpos-1");
        }, 600);
    };

    /**
     * Moves the lockpick to a new position.
     * @param {int} newPos The new position to move to. A number from 0 to 4.
     */
    this.moveToPos = function (newPos) {
        ctrl.set(newPos);
        var currentClass = positionToClass();
        that.position = newPos;
        var newClass = positionToClass();
        swapClasses(currentClass, newClass);
    };
}
