/**
 * Class capable of importing single saves or entire save lists from file.
 * Does not distinguish between lists and single saves.
 */
function SaveImporter() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var fileErrorCached = false;
    var saveLimitErrorCached = false;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Caches a file error.
     */
    function fileError() {
        fileErrorCached = true;
    }

    /**
     * Caches a save limit error.
     */
    function saveLimitError() {
        saveLimitErrorCached = true;
    }

    /**
     * Resolves any cached file reading errors.
     */
    function resolveFileError() {
        if (fileErrorCached) {
            fileErrorCached = false;
            alert("One or more of the selected files failed to parse. They may be corrupted.");
        }
    }

    /**
     * Resolves any cached save limit errors.
     */
    function resolveSaveLimitError() {
        if (saveLimitErrorCached) {
            saveLimitErrorCached = false;
            alert("The limit of total saves has been reached. Some saves may not have been added!");
        }
    }

    /**
     * Resolves all error caches, telling the user what went wrong (if anything).
     */
    function resolveErrorCaches() {
        resolveFileError();
        resolveSaveLimitError();
    }

    /**
     * Attempts to turn the passed text into a JSON-parsed array.
     * Shows an error message if this fails.
     * @param {string} text File-read text.
     * @returns {Array} The parsed array. Empty if an error occured.
     */
    function textToArray(text) {
        var array;
        try {
            array = JSON.parse(text);
        } catch (e) {
            fileError();
            return [];
        }
        if (!array || !Array.isArray(array)) {
            fileError();
            return [];
        }
        return array;
    }

    /**
     * Adds the passed object representing a save to the game's list of saves.
     * @param {object} object The object representing a save. Will be parsed to JSON.
     */
    function save(object) {
        var slot = global.saveManager.getFirstOpenSaveSlot();
        if (slot < 0) {
            saveLimitError();
            return;
        }
        var json = JSON.stringify(object);
        global.saveManager.jsonToSlot(slot, json);
    }

    /**
     * Extracts the saves from the text, which is assumed to be a JSON-array.
     * Shows an alert when the save is malformed.
     * @param {string} text The text to extract saves from. Assumed to be a JSON-array.
     */
    function saves(text) {
        var array = textToArray(text);
        for (var i in array) {
            save(array[i]);
        }
    }

    /**
     * Called when the filereader is done reading the file.
     * Will extract the text read from the file and pass it to the save creation function.
     * @param {event} event The done reading event.
     */
    function doneReading(event) {
        var text = event.target.result;
        saves(text);
    }

    /**
     * Imports a save file.
     * Shows a file dialogue to pick the file.
     * @param {File} toRead A single file taken from a FileList produced by an input event.
     * @returns {Promise} A promise that will resolve when the file has been read and parsed.
     */
    function file(toRead) {
        var reader = new FileReader();
        var promiseResolve;
        var promise = new Promise(function (resolve, reject) {
            promiseResolve = resolve;
        });
        reader.onload = function (event) {
            doneReading(event);
            promiseResolve();
        };
        reader.readAsText(toRead);
        return promise;
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Imports multiple save files.
     * Shows a file dialogue to pick the file.
     * @param {FileList} fileList The FileList produced by the input event.
     * @param {function} fun Callback function. Gets called after every file has been read.
     */
    this.files = function (fileList, fun) {
        if (!window.File || !window.FileReader || !window.FileList || !window.Blob) {
            alert("Browser does not support file importing.");
            return;
        }
        var loadingPromises = [];
        for (var i = 0; i < fileList.length; i++) {
            loadingPromises.push(file(fileList[i]));
        }
        Promise.all(loadingPromises).then(function () {
            resolveErrorCaches();
            fun && fun();
        });
    };

    this.test = file;

}
