/**
 * Saved section of the home scene.
 * @param {HomeController} controller The home controller.
 */
function HomeSaved(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var hideId = "savedHideButton";
    var talkId = "savedTalkButton";
    var ambushId = "savedAmbushButton";

    var toHide = new HomeHide(ctrl);
    var toTalk = new HomeApproach(ctrl);
    var toAmbush = new HomeAmbush(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[st]Early arrival[/st]`;
        toReturn += `   [p]
                            After a short, relaxing trek through the forest,
                            you begin to near your small hovel,
                            located near the northern edge of the woods.
                            You've been living there for just about as long as you can remember.
                            You think it used to serve as a storage room for a larger building,
                            seeing as the somewhat concealed dirt track leading towards your home abruptly ends a good bit away from the actual shack.
                        [/p]
                        [p]
                            As your shed comes into view, you spot a shady figure sitting alongside it on the tree stump that has been serving as your outside-stool.
                            The person in question is a kobold like you, but even from a distance you can see that he's [b]far[/b] larger than you are,
                            in both size and in [b]girth[/b].
                            A crossbreed between your race and something [i]bigger[/i], or so you figure.
                            Probably a pig, judging by his [i]size[/i].
                        [/p]
                        [p]
                            The half-kobold is wearing a ${global.kobold.inventory.createItemWithTooltip(new StuddedLeatherChestPiece())}
                            with matching, equally combat-ready ${global.kobold.inventory.createItemWithTooltip(new LeatherPants())}.
                            His attire makes you think that he was sent to collect your [i]debt[/i], by any means necessary.
                            He isn't visibly armed, but you doubt someone of his [b]bulk[/b] needs anything other than his oversized fists to beat the shit out of you.
                        [/p]
                        [p]
                            The pigbold seemingly hasn't noticed you yet.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the sneak choice button.
     */
    function buildHideChoice() {
        var disabled = ctrl.saved.hasMadeApproachChoice() ? "disabled" : "";
        var isSelected = ctrl.saved.getApproachChoice() === HomeSavedChoice.HIDE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${hideId}" ${disabled}>Hide your gold</button>`;

        var tooltip = `Stash your gold in a nearby bush before approaching.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the talk choice button.
     */
    function buildTalkChoice() {
        var disabled = ctrl.saved.hasMadeApproachChoice() ? "disabled" : "";
        var isSelected = ctrl.saved.getApproachChoice() === HomeSavedChoice.TALK;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${talkId}" ${disabled}>Talk to him</button>`;

        var tooltip = `Approach the debt collector and talk to him.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 175px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the ambush choice button.
     */
    function buildAmbushChoice() {
        var disabled = ctrl.saved.hasMadeApproachChoice() ? "disabled" : "";
        var isSelected = ctrl.saved.getApproachChoice() === HomeSavedChoice.AMBUSH;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${ambushId}" ${disabled}>Ambush him</button>`;

        var tooltip = `Sneak up from behind and try to ambush the kobold.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 185px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildHideChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildTalkChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildAmbushChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + hideId), $("#" + talkId), $("#" + ambushId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerHideListener() {
        $("#" + hideId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.saved.makeApproachChoice(HomeSavedChoice.HIDE);
            loadNextPart(toHide);
        });

    }

    /**
     * Adds a listener to the button.
     */
    function registerTalkListener() {
        $("#" + talkId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.saved.makeApproachChoice(HomeSavedChoice.TALK);
            loadNextPart(toTalk);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerAmbushListener() {
        $("#" + ambushId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.saved.makeApproachChoice(HomeSavedChoice.AMBUSH);
            loadNextPart(toAmbush);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "homeSaved";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.status() !== HomeStatusEnum.SAVED) {
            return "";
        }
        if (!ctrl.saved.hasMadeApproachChoice()) {
            ctrl.introAudio.playSettingsDependent();
        }
        global.ui.applyTheme("home");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += toHide.build();
        part += toTalk.build();
        part += toAmbush.build();
        part += `</div>`;

        // BUILD CHILDREN HERE

        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.status() !== HomeStatusEnum.SAVED) {
            return;
        }

        toHide.registerListeners();
        toTalk.registerListeners();
        toAmbush.registerListeners();

        if (ctrl.saved.hasMadeApproachChoice()) {
            return;
        }

        registerHideListener();
        registerTalkListener();
        registerAmbushListener();
    };
}

var HomeSavedChoice = {
    HIDE: 1,
    TALK: 2,
    AMBUSH: 3
};
